@@ -1,3 +1,4 @@
+from openwisp_users.api.mixins import FilterSerializerByOrgManaged
 from openwisp_utils.api.serializers import ValidatedModelSerializer
 from rest_framework import serializers
 from swapper import load_model
@@ -13,14 +14,14 @@ class IpRequestSerializer(ValidatedModelSerializer):
         read_only_fields = ('created', 'modified')
 
 
-class IpAddressSerializer(ValidatedModelSerializer):
+class IpAddressSerializer(FilterSerializerByOrgManaged, ValidatedModelSerializer):
     class Meta:
         model = IpAddress
         fields = '__all__'
         read_only_fields = ('created', 'modified')
 
 
-class SubnetSerializer(ValidatedModelSerializer):
+class SubnetSerializer(FilterSerializerByOrgManaged, ValidatedModelSerializer):
     class Meta:
         model = Subnet
         fields = '__all__'
