@@ -1,12 +1,12 @@
 import json
 import os
 
-from openwisp_users.models import Organization
 from openwisp_users.tests.utils import TestOrganizationMixin
 from swapper import load_model
 
 Subnet = load_model('openwisp_ipam', 'Subnet')
 IpAddress = load_model('openwisp_ipam', 'IpAddress')
+Organization = load_model('openwisp_users', 'Organization')
 
 
 class FileMixin(object):
@@ -22,7 +22,11 @@ class CreateModelsMixin(TestOrganizationMixin):
         return kwargs
 
     def _create_subnet(self, **kwargs):
-        options = dict(subnet='', description='',)
+        options = dict(
+            name='test subnet',
+            subnet='',
+            description='',
+        )
         options.update(self._get_extra_fields(**kwargs))
         options.update(kwargs)
         instance = Subnet(**options)
@@ -31,7 +35,10 @@ class CreateModelsMixin(TestOrganizationMixin):
         return instance
 
     def _create_ipaddress(self, **kwargs):
-        options = dict(ip_address='', description='',)
+        options = dict(
+            ip_address='',
+            description='',
+        )
         options.update(kwargs)
         instance = IpAddress(**options)
         instance.full_clean()
