@@ -1,8 +1,7 @@
-from django.conf.urls import url
 from django.urls import include, path
-from openwisp_users.api.urls import get_api_urls as get_users_api_urls
 
-from .api import get_api_urls, views as ipam_api_views
+from .api import views as ipam_api_views
+from .api.urls import get_api_urls
 
 
 def get_urls(api_views):
@@ -12,9 +11,10 @@ def get_urls(api_views):
         api_views: location for getting API views
     """
     return [
-        path('api/v1/', include((get_api_urls(api_views), 'ipam'), namespace='ipam')),
+        path(
+            'api/v1/ipam/', include((get_api_urls(api_views), 'ipam'), namespace='ipam')
+        ),
         path('accounts/', include('openwisp_users.accounts.urls')),
-        url(r'^api/v1/', include((get_users_api_urls(), 'users'), namespace='users')),
     ]
 
 
